/*
 * Chax_AbstractAccount.m
 *
 * Copyright (c) 2007-2009 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "Chax_AbstractAccount.h"
#import "Chax.h"
#import "StatusChangeController.h"
#import "UnifiedPeopleListController.h"
#import "UnifiedAccount.h"
#import "Chax_ActiveChat.h"

@implementation AbstractAccount (Chax)

+ (void)load
{
	MethodSwizzle([self class], @selector(_createChatWithID:message:style:isInvitationMessage:), @selector(chax_createChatWithID:message:style:isInvitationMessage:));
	MethodSwizzle([self class], @selector(invitedToChat:isChatRoom:invitation:), @selector(chax_invitedToChat:isChatRoom:invitation:));
	MethodSwizzle([self class], @selector(_handleChat:messageReceived:), @selector(chax_handleChat:messageReceived:));
	MethodSwizzle([self class], @selector(renameGroup:to:), @selector(chax_renameGroup:to:));
	MethodSwizzle([self class], @selector(loginStatusChanged:message:reason:properties:), @selector(chax_loginStatusChanged:message:reason:properties:));
}

- (id)chax_createChatWithID:(id)fp8 message:(id)fp12 style:(int)fp16 isInvitationMessage:(BOOL)fp20
{
	id chat = [self chax_createChatWithID:fp8 message:fp12 style:fp16 isInvitationMessage:fp20];
	
	[chat chax_appendChatHistory:YES];
	
	return chat;
}

- (void)chax_loginStatusChanged:(unsigned int)fp8 message:(id)fp12 reason:(unsigned int)fp16 properties:(id)fp20
{
	[self chax_loginStatusChanged:fp8 message:fp12 reason:fp16 properties:fp20];
	
	if (fp8 == 0) {
		[[UnifiedPeopleListController sharedController] reloadContacts];
	}
	
	[[UnifiedPeopleListController sharedController] rebuildAddBuddyMenu];
}

- (void)chax_invitedToChat:(id)fp8 isChatRoom:(BOOL)fp12 invitation:(id)fp16
{
	if ([[(NSString *)[fp16 sender] lowercaseString] isEqualToString:@"aolsystemmsg"] && [Chax boolForKey:@"BlockAOLSystemMessage"]) {
		return;
	}
	
	[self chax_invitedToChat:fp8 isChatRoom:fp12 invitation:fp16];
	
	if (![NSApp isActive]) {
		InstantMessage *message = [self _createInstantMessage:fp16 chatID:fp8];
		Presentity *presentity = [message sender];
		
		if ([Chax boolForKey:@"GrowlEnabled"] && [[[message text] string] length] > 0) {
			[[StatusChangeController sharedController] postGrowlNotificationWithTitle:[NSString stringWithFormat:ChaxLocalizedString(@"growl_says"), [presentity name]]
										description:[[message text] string]
										notificationName:ChaxGrowlTextInvitation
										iconData:[[[presentity picture] image] TIFFRepresentation]
										clickContext:[NSDictionary dictionaryWithObject:fp8 forKey:@"Chat"]];
		}
	}
	
	if ([Chax boolForKey:@"StopScreensaverOnNewIM"]) {
		[Chax undimScreenAndStopScreenSaver];
	}
}

- (void)chax_handleChat:(id)fp8 messageReceived:(id)fp12
{
	[self chax_handleChat:fp8 messageReceived:fp12];
	
	if (![NSApp isActive]) {
		InstantMessage *message = [self _createInstantMessage:fp12 chatID:fp8];
		Presentity *presentity = [self existingPresentityWithID:[fp12 sender]];	
		
		//Post notification only if the message was received from another user, and if there was an associated message
		if ([Chax boolForKey:@"GrowlEnabled"] && [fp12 flags] == 1 && [[[message text] string] length] > 0) {
			[[StatusChangeController sharedController] postGrowlNotificationWithTitle:[NSString stringWithFormat:ChaxLocalizedString(@"growl_says"), [presentity name]]
										description:[[message text] string]
										notificationName:ChaxGrowlNewMessage
										iconData:[[[presentity picture] image] TIFFRepresentation]
										clickContext:[NSDictionary dictionaryWithObject:fp8 forKey:@"Chat"]];
		}
	}
	
	if ([fp12 flags] == 1) {
		if ([Chax boolForKey:@"StopScreensaverOnIM"]) {
			[Chax undimScreenAndStopScreenSaver];
		}
	}
}

- (void)chax_renameGroup:(id)fp8 to:(id)fp12
{
	[self chax_renameGroup:fp8 to:fp12];
	
	[[UnifiedPeopleListController sharedController] performSelector:@selector(_serviceGroupsChanged:) withObject:nil afterDelay:0.0];
}

/*- (void)chax_serviceDidDisconnect:(id)fp8
{
	[self chax_serviceDidDisconnect:fp8];
	
	if (fp8 && [Chax boolForKey:@"AttemptAutoReconnect"]) {
		[self performSelector:@selector(chax_attemptReconnect) withObject:nil afterDelay:5.0];
	}
}

- (void)chax_attemptReconnect
{
	//This doesn't work
	if (![self loggedIn]) {
		//NSLog(@"Attempting to reconnect: %@", self);
		[self login];
	}
}*/

@end
