/*
 * Chax_ActiveChat.m
 *
 * Copyright (c) 2007-2009 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "Chax_ActiveChat.h"
#import "Chax_AIMLGenerator.h"
#import "Chax.h"

@implementation ActiveChat (Chax)

+ (void)load
{
	MethodSwizzle([self class], @selector(_autoreplyToMsg:whenMyStatusIs:), @selector(chax_autoreplyToMsg:whenMyStatusIs:));
	MethodSwizzle([self class], @selector(finish), @selector(chax_finish));
}

- (void)chax_appendChatHistory:(BOOL)isIncoming
{
	if ([Chax boolForKey:@"ShowMessageHistory"]) {
		//Check if there's a saved transcript path
		NSString *transcriptPath = [[Chax objectForKey:@"RecentSavedChats"] objectForKey:[[self otherPresentity] ID]];
		
		if (transcriptPath) {
			SavedChat *savedChat = [[SavedChat alloc] initWithTranscriptFile:transcriptPath];
			NSArray *messages = [savedChat messages];
			int messagesAdded = 0;
			NSMutableArray *messagesToAdd = [NSMutableArray array];
			
			/*NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
			[formatter setDateStyle:NSDateFormatterShortStyle];
			[formatter setTimeStyle:NSDateFormatterShortStyle];
			[self addAnnouncementString:[formatter stringFromDate:[savedChat dateCreated]]];
			[formatter release];*/
			
			for (int i = [messages count] - 1; i >= 0 && messagesAdded < 5; i--) {
				InstantMessage *nextMessage = [messages objectAtIndex:i];
				if ([nextMessage sender]) {
					[messagesToAdd insertObject:nextMessage atIndex:0];
					messagesAdded++;
				}
			}
			
			BOOL first = YES;
			InstantMessage *savedMessage = nil;
			
			for (InstantMessage *message in messagesToAdd) {
				//if (isIncoming && first) {
				if (first) {
					savedMessage = [[_messages objectAtIndex:0] retain];
					[self addMessage:message atEnd:NO replacingMessageAtIndex:0];
					first = NO;
				} else {
					[self addMessage:message];
				}
			}
			
			[self addAnnouncementString:@"______________"];
			//NSDictionary *dictionary = [NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:YES], @"atEnd", [_messages lastObject], @"newMessage", nil];
			//[[NSNotificationCenter defaultCenter] postNotificationName:@"ChatMessagesChanged" object:self userInfo:dictionary];
			
			[self _setStartingIndex:[[self messages] count]];
			
			if (savedMessage) {
				[self addMessage:savedMessage];
				[savedMessage release];
			}
			
			/*if (!isIncoming) {
				[self addInitialComposeMessage];
			}*/
			
			[savedChat release];
		}
	}
}

- (void)chax_autoreplyToMsg:(id)fp8 whenMyStatusIs:(unsigned int)fp12
{
	//If we're logged in through ICQ and we're talking to an ICQ user flag the AIML generator to send plain text
	if ([[[self otherPresentity] ID] isICQ] && ![Chax boolForKey:@"DisablePlainText"]) {
		[Chax_AIMLGenerator chax_nextMessagePlainText];
	}
	
	[self chax_autoreplyToMsg:fp8 whenMyStatusIs:fp12];
}

- (void)chax_finish
{
	[self chax_finish];
	
	if ([Chax boolForKey:@"ShowMessageHistory"]) {
		NSString *dateFolder = [[self dateCreated] descriptionWithCalendarFormat:NSLocalizedString(@"%Y-%m-%d", @"%Y-%m-%d") timeZone:nil locale:nil];
		NSString *savedChatPath = [[[[Prefs savedChatPath] stringByAppendingPathComponent:dateFolder] stringByAppendingPathComponent:[self defaultSaveName]] stringByAppendingPathExtension:@"ichat"];
		
		if ([[NSFileManager defaultManager] fileExistsAtPath:savedChatPath]) {
			//Check if there is a #2 chat
			if ([[NSFileManager defaultManager] fileExistsAtPath:[[savedChatPath stringByDeletingPathExtension] stringByAppendingString:@" #2.ichat"]]) {
				savedChatPath = [[savedChatPath stringByDeletingPathExtension] stringByAppendingString:@" #2.ichat"];
			}
			
			NSMutableDictionary *savedChats = [[[Chax objectForKey:@"RecentSavedChats"] mutableCopy] autorelease];
			if (!savedChats) {
				savedChats = [NSMutableDictionary dictionary];
			}
			
			[savedChats setObject:savedChatPath forKey:[[self otherPresentity] ID]];
			[Chax setObject:savedChats forKey:@"RecentSavedChats"];
		}
	}
}

@end
