/*
 * Chax_AnimatingTableView.m
 *
 * Copyright (c) 2007-2009 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "Chax_AnimatingTableView.h"
#import "Chax.h"
#import "LogViewerController.h"

#define VIEW_LOGS_TAG	91120

@implementation AnimatingTableView (Chax)

+ (void)load
{
	MethodSwizzle([self class], @selector(menuForEvent:), @selector(chax_menuForEvent:));
	MethodSwizzle([self class], @selector(draggingExited:), @selector(chax_draggingExited:));
	MethodSwizzle([self class], @selector(performDragOperation:), @selector(chax_performDragOperation:));
}

- (void)chax_draggingExited:(id <NSDraggingInfo>)sender
{
	[[self class] cancelPreviousPerformRequestsWithTarget:self selector:@selector(chax_popToFront) object:nil];
	[self chax_draggingExited:sender];
}

- (BOOL)chax_performDragOperation:(id)fp8
{
	[[self class] cancelPreviousPerformRequestsWithTarget:self selector:@selector(chax_popToFront) object:nil];
	return [self chax_performDragOperation:fp8];
}

- (NSMenu *)chax_menuForEvent:(NSEvent *)event
{
	NSMenu *menu = [self chax_menuForEvent:event];
	if ([menu numberOfItems] > 3) {
		NSMenuItem *menuItem;
		
		//Remove the previous "More Info" and "View Logs" menu items
		menuItem = [menu itemWithTag:VIEW_LOGS_TAG];
		if (menuItem) {
			[menu removeItem:menuItem];
		}
		
		menuItem = [menu addItemWithTitle:ChaxLocalizedString(@"view_logs") action:@selector(chax_menuAction:) keyEquivalent:@""];
		[menuItem setTag:VIEW_LOGS_TAG];
		[menuItem setTarget:self];
		[menuItem setRepresentedObject:[NSNumber numberWithInt:[self _rowAtEventLocation:event]]];
		
		if ([Chax boolForKey:@"ShowMenuBadge"]) {
			NSImage *badgeImage = [[[NSImage alloc] initByReferencingFile:[[NSBundle bundleWithIdentifier:@"com.ksuther.chax"] pathForImageResource:@"menu-badge"]] autorelease];
			[menuItem setImage:badgeImage];
		}
	}
	return menu;
}

- (void)chax_menuAction:(id)sender
{
	switch ([sender tag]) {
		case VIEW_LOGS_TAG:
			[[LogViewerController sharedController] window];
			[[LogViewerController sharedController] showLogsForPresentity:[[self delegate] presentityAtRow:[[sender representedObject] intValue]]];
	}
}

- (void)scrollWheel:(NSEvent *)event
{
	if ([[self delegate] isKindOfClass:[ChatWindowController class]] && [Chax boolForKey:@"ScrollWheelSwitchesChats"]) {
		if ([event deltaY] > 0) {
			[[self delegate] gotoPreviousChat:nil];
		} else {
			[[self delegate] gotoNextChat:nil];
		}
	} else {
		[super scrollWheel:event];
	}
}

- (void)chax_popToFront
{
	[[self window] setLevel:NSStatusWindowLevel];
	[[self window] setLevel:NSNormalWindowLevel];
}

@end
