/*
 * Chax_Chat.m
 *
 * Copyright (c) 2007-2009 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "Chax_Chat.h"
#import "Chax.h"
#import "DockIconController.h"

static NSMutableDictionary *_chaxCreationDates = nil;

@implementation Chat (Chax)

+ (void)load
{
	MethodSwizzle([self class], @selector(initWithAccount:style:), @selector(chax_initWithAccount:style:));
	MethodSwizzle([self class], @selector(dealloc), @selector(chax_dealloc));
	MethodSwizzle([self class], @selector(dateCreated), @selector(chax_dateCreated));
	MethodSwizzle([self class], @selector(numberOfUnreadMessages), @selector(chax_numberOfUnreadMessages));
}

- (id)chax_initWithAccount:(id)fp8 style:(int)fp12
{
	id chat = [self chax_initWithAccount:fp8 style:fp12];
	
	if (!_chaxCreationDates) {
		_chaxCreationDates = [[NSMutableDictionary alloc] init];
	}
	
	[_chaxCreationDates setObject:[NSDate date] forKey:[NSValue valueWithPointer:chat]];
	
	return chat;
}

- (void)chax_dealloc
{
	[_chaxCreationDates removeObjectForKey:[NSValue valueWithPointer:self]];
	[self chax_dealloc];
}

- (id)chax_dateCreated
{
	NSDate *date = [_chaxCreationDates objectForKey:[NSValue valueWithPointer:self]];
	//Message history can screw with things, so store the creation date internally
	return date ? date : [self chax_dateCreated];
}

- (int)chax_numberOfUnreadMessages
{
	int unreadCount = [self chax_numberOfUnreadMessages];
	
	if (unreadCount > 0 && (self != [[self chatWindowController] currentChat] || ![NSApp isActive])) {
		[[DockIconController sharedController] addChat:self];
	}
	
	return unreadCount;
}

@end
