/*
 * Chax_ChatDOMView.m
 *
 * Copyright (c) 2007-2009 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "Chax_ChatDOMView.h"
#import "Chax.h"
#import "DockIconController.h"

static BOOL _quickLookLoadFailed = NO;
static BOOL _quickLookLoaded = NO;
static NSRect _nextQLRect;

@implementation ChatDOMView (Chax)

+ (void)load
{
	MethodSwizzle([self class], @selector(dealloc), @selector(chax_dealloc));
	MethodSwizzle([self class], @selector(_infoBannerForMessage:), @selector(chax_infoBannerForMessage:));
	MethodSwizzle([self class], @selector(clearUnreadMessageIfVisible), @selector(chax_clearUnreadMessageIfVisible));
	MethodSwizzle([self class], @selector(webView:contextMenuItemsForElement:defaultMenuItems:), @selector(chax_webView:contextMenuItemsForElement:defaultMenuItems:));
	
	_nextQLRect = NSZeroRect;
}

- (void)chax_dealloc
{
	if ([[[QLPreviewPanel sharedPreviewPanel] windowController] delegate] == self) {
		[[[QLPreviewPanel sharedPreviewPanel] windowController] setDelegate:nil];
	}
	
	[self chax_dealloc];
}

- (id)chax_infoBannerForMessage:(id)fp8
{
	id banner = [self chax_infoBannerForMessage:fp8];
	return ([Chax boolForKey:@"ShowSenderAccount"] && [(Presentity *)[fp8 sender] account] != nil) ? [banner stringByAppendingFormat:ChaxLocalizedString(@"im_from"), [(Presentity *)[fp8 sender] account]] : banner;
}

- (void)chax_clearUnreadMessageIfVisible
{
	if (![[self chat] hasUnreadMessages]) {
		[[DockIconController sharedController] removeChat:[self chat]];
	}
	[self chax_clearUnreadMessageIfVisible];
}

- (id)chax_webView:(id)fp8 contextMenuItemsForElement:(id)fp12 defaultMenuItems:(id)fp16
{
	NSURL *imageURL;
	
	if (!_quickLookLoadFailed && (imageURL = [fp12 objectForKey:@"WebElementImageURL"]) && [imageURL isFileURL]) {
		//Try to load the QuickLookUI framework if it isn't loadaed
		if (!_quickLookLoaded) {
			if ([[NSBundle bundleWithPath:@"/System/Library/PrivateFrameworks/QuickLookUI.framework"] load]) {
				_quickLookLoaded = YES;
			} else {
				NSLog(@"Error loading QuickLookUI.framework");
				_quickLookLoadFailed = YES;
			}
		}
		
		if (_quickLookLoaded) {
			NSMutableArray *items = [[[self chax_webView:fp8 contextMenuItemsForElement:fp12 defaultMenuItems:fp16] mutableCopy] autorelease];
			NSString *menuTitle = ChaxLocalizedString([[QLPreviewPanel sharedPreviewPanel] isVisible] ? @"close_quick_look" : @"view_with_quick_look");
			NSMenuItem *menuItem = [[[NSMenuItem alloc] initWithTitle:menuTitle action:@selector(chax_quickLookImage:) keyEquivalent:@""] autorelease];
			
			[items addObject:menuItem];
			[menuItem setTarget:self];
			
			if ([Chax boolForKey:@"ShowMenuBadge"]) {
				NSImage *badgeImage = [[[NSImage alloc] initByReferencingFile:[[NSBundle bundleWithIdentifier:@"com.ksuther.chax"] pathForImageResource:@"menu-badge"]] autorelease];
				[menuItem setImage:badgeImage];
			}
			
			if (![[QLPreviewPanel sharedPreviewPanel] isVisible]) {
				[menuItem setRepresentedObject:imageURL];
				
				//Find the rect of the chosen image in screen coordinates
				_nextQLRect = [[[[self mainFrame] frameView] documentView] convertRect:[[fp12 objectForKey:@"WebElementImageRect"] rectValue] toView:nil];
				NSPoint screenOrigin = [[self window] convertBaseToScreen:_nextQLRect.origin];
				_nextQLRect.origin = screenOrigin;
			}
			
			return items;
		}
	}
	
	return [self chax_webView:fp8 contextMenuItemsForElement:fp12 defaultMenuItems:fp16];
}

- (void)chax_quickLookImage:(NSMenuItem *)sender
{
	if ([[QLPreviewPanel sharedPreviewPanel] isVisible]) {
		[[QLPreviewPanel sharedPreviewPanel] closeWithEffect:2];
	} else {
		[NSApp activateIgnoringOtherApps:YES];
		[[QLPreviewPanel sharedPreviewPanel] setURLs:[NSArray arrayWithObject:[sender representedObject]] currentIndex:0 preservingDisplayState:YES];
		[[[QLPreviewPanel sharedPreviewPanel] windowController] setDelegate:self];
		[[QLPreviewPanel sharedPreviewPanel] makeKeyAndOrderFrontWithEffect:2];
	}
}

- (NSRect)previewPanel:(NSPanel*)panel frameForURL:(NSURL*)URL
{
	return _nextQLRect;
}

@end
