/*
 * Chax_ChatWindowController.m
 *
 * Copyright (c) 2007-2009 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "Chax_ChatWindowController.h"
#import "Chax.h"
#import "Chax_ActiveChat.h"
#import "CameraSnapshotController.h"

Chat *_lastChat = nil;

@implementation ChatWindowController (Chax)

+ (void)load
{
	MethodSwizzleClass([self class], @selector(displayChatForPresentity:style:), @selector(chax_displayChatForPresentity:style:));
	MethodSwizzle([self class], @selector(windowDidLoad), @selector(chax_windowDidLoad));
	MethodSwizzle([self class], @selector(addChatInvite:withNotifierWindow:), @selector(chax_addChatInvite:withNotifierWindow:));
	MethodSwizzle([self class], @selector(selectChat:), @selector(chax_selectChat:));
	MethodSwizzle([self class], @selector(closeChat:), @selector(chax_closeChat:));
	MethodSwizzle([self class], @selector(validateMenuItem:), @selector(chax_validateMenuItem:));
}

+ (id)chax_displayChatForPresentity:(id)fp8 style:(int)fp12
{
	id chat = [self chax_displayChatForPresentity:fp8 style:fp12];
	
	if ([chat isComposing]) {
		[chat chax_appendChatHistory:NO];
	}
	
	return chat;
}

- (void)chax_windowDidLoad
{
	[self chax_windowDidLoad];
	
	if ([Chax boolForKey:@"HideChatsWhenInactive"]) {
		[[self window] setHidesOnDeactivate:YES];
	}
}

- (void)chax_addChatInvite:(id)fp8 withNotifierWindow:(id)fp12
{
	_lastChat = fp8;
	[self chax_addChatInvite:fp8 withNotifierWindow:fp12];
}

- (void)chax_selectChat:(id)fp8
{
	if ([Chax boolForKey:@"SkipNewMessageNotification"] && _lastChat && _lastChat == fp8) {
		[self performSelector:@selector(chax_allowSelect) withObject:nil afterDelay:0.0];
	} else {
		[self chax_selectChat:fp8];
	}
}

- (void)chax_allowSelect
{
	_lastChat = nil;
}

- (void)chax_closeChat:(id)fp8
{
	if (![ChatWindowController wantsCollectedChats] && [Chax boolForKey:@"SaveWindowFrames"]) {
		NSMutableDictionary *presentities = [[Chax objectForKey:@"CustomPresentities"] mutableCopy];
		NSString *key = [[[[self currentChat] otherPresentity] abPerson] uniqueId];
		NSString *userID = [[[[self currentChat] otherPresentity] ID] lowercaseString];
		NSString *frameString = [[self window] stringWithSavedFrame];
		NSDictionary *info;
		NSMutableDictionary *mutableInfo;
		
		if (presentities == nil) {
			presentities = [[NSMutableDictionary alloc] init];
		}
		
		if (key == nil) {
			key = userID;
		}
		
		info = [presentities objectForKey:key];
		
		if (userID) {
			if (info) {
				mutableInfo = [info mutableCopy];
				[mutableInfo setObject:frameString forKey:[@"WindowFrame." stringByAppendingString:userID]];
				[presentities setObject:mutableInfo forKey:key];
				[mutableInfo release];
			} else {
				[presentities setObject:[NSDictionary dictionaryWithObject:frameString forKey:[@"WindowFrame." stringByAppendingString:userID]] forKey:key];
			}
			[Chax setObject:presentities forKey:@"CustomPresentities"];
		}
		
		[presentities release];
	}
	
	[self chax_closeChat:fp8];
}

- (BOOL)chax_validateMenuItem:(NSMenuItem *)menuItem
{
	BOOL valid = [self chax_validateMenuItem:menuItem];
	
	if ([menuItem tag] == SEND_CAMERA_SNAPSHOT_MENU_ITEM) {
		valid = [[[self currentChatController] fieldEditor] canAttachFile];
	}
	
	return valid;
}

- (void)chax_sendCameraSnapshot:(id)sender
{
	CameraSnapshotController *controller = [[CameraSnapshotController alloc] init];
	
	controller.delegate = self;
	[controller showWindow:nil];
}

#pragma mark -
#pragma mark CameraSnapshotController Delegate

- (void)cameraSnapshotControllerDidCancel:(CameraSnapshotController *)cameraSnapshotController
{
	[cameraSnapshotController release];
}

- (void)cameraSnapshotController:(CameraSnapshotController *)cameraSnapshotController didTakeSnapshot:(NSImage *)image
{
	[[[self currentChatController] fieldEditor] insertAttachedFile:[image temporaryJPEGFileWithCompression:60]];
	//[(ChatController *)[self currentChatController] sendMessage:nil];
	
	[cameraSnapshotController release];
}

@end
