/*
 * Chax_Fezz.m
 *
 * Copyright (c) 2007-2009 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "Chax_Fezz.h"
#import "Chax.h"
#import "LogViewerController.h"
#import "UnifiedPeopleListController.h"

@implementation Fezz (Chax)

+ (void)load
{
	MethodSwizzle([self class], @selector(applicationShouldHandleReopen:hasVisibleWindows:), @selector(chax_applicationShouldHandleReopen:hasVisibleWindows:));
	MethodSwizzle([self class], @selector(applicationShouldTerminate:), @selector(chax_applicationShouldTerminate:));
	MethodSwizzle([self class], @selector(_persistCurrentChats), @selector(chax_persistCurrentChats));
	MethodSwizzle([self class], @selector(_handleGURLAppleEvent:reply:), @selector(chax_handleGURLAppleEvent:reply:));
}

- (BOOL)chax_applicationShouldHandleReopen:(id)fp8 hasVisibleWindows:(BOOL)fp12
{
	if (!fp12 && [Chax boolForKey:@"PreferAllContacts"]) {
		[[UnifiedPeopleListController sharedController] showWindow:nil];
		return NO;
	}
	
	return [self chax_applicationShouldHandleReopen:fp8 hasVisibleWindows:fp12];
}

- (NSApplicationTerminateReply)chax_applicationShouldTerminate:(NSApplication *)sender
{
	NSApplicationTerminateReply reply;
	
	if ([Chax boolForKey:@"ConfirmQuit"] && [[ChatWindowController allChatWindowControllers] count] > 0) {
		NSAlert *alert = [[NSAlert alloc] init];
		[alert setMessageText:NSLocalizedString(@"There are active chats. Are you sure you want to quit?", @"There are active chats. Are you sure you want to quit?")];
		[alert addButtonWithTitle:NSLocalizedString(@"Yes", @"Yes")];
		[[alert addButtonWithTitle:NSLocalizedString(@"No", @"No")] setKeyEquivalent:@"\e"];
		
		reply = ([alert runModal] == NSAlertFirstButtonReturn) ? : NSTerminateCancel;
		[alert release];
	} else {
		reply = [self chax_applicationShouldTerminate:sender];
	}
	
	return reply;
}

- (void)chax_persistCurrentChats
{
	[[LogViewerController sharedController] clearChatView];
	[self chax_persistCurrentChats];
}

- (void)chax_handleGURLAppleEvent:(id)fp8 reply:(id)fp12
{
	NSRange range = [[[fp8 descriptorAtIndex:1] stringValue] rangeOfString:@"iChat:openbuddylist?service="];
	
	if ([Chax boolForKey:@"PreferAllContacts"] && range.location != NSNotFound) {
		[[UnifiedPeopleListController sharedController] showWindow:nil];
	} else {
		[self chax_handleGURLAppleEvent:fp8 reply:fp12];
	}
}

@end
