/*
 * Chax_People.m
 *
 * Copyright (c) 2007-2009 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "Chax_People.h"
#import "Chax.h"
#import "UnifiedPeopleListController.h"

@implementation People (Chax)

+ (void)load
{
	MethodSwizzle([self class], @selector(addPresentity:), @selector(chax_addPresentity:));
	MethodSwizzle([self class], @selector(addPeopleFromArray:), @selector(chax_addPeopleFromArray:));
	MethodSwizzle([self class], @selector(removePresentity:), @selector(chax_removePresentity:));
	MethodSwizzle([self class], @selector(removePeopleFromArray:), @selector(chax_removePeopleFromArray:));
}

- (BOOL)chax_addPresentity:(id)fp8
{
	NSArray *existingPresentities = [[[UnifiedPeopleListController sharedController] sourcePeople] people];
	
	for (Presentity *presentity in existingPresentities) {
		if ([[presentity canonicalFormOfID] isEqualToString:[fp8 canonicalFormOfID]]) {
			//Don't add the presentity if the same user was already added on another service
			return [self chax_addPresentity:fp8];
		}
	}
	
	[[[UnifiedPeopleListController sharedController] sourcePeople] chax_addPresentity:fp8];
	return [self chax_addPresentity:fp8];
}

- (BOOL)chax_addPeopleFromArray:(id)fp8
{
	[[[UnifiedPeopleListController sharedController] sourcePeople] chax_addPeopleFromArray:fp8];
	return [self chax_addPeopleFromArray:fp8];
}

- (BOOL)chax_removePresentity:(id)fp8
{
	NSArray *controllers = [PeopleListController peopleListControllers];
	
	for (PeopleListController *plc in controllers) {
		if (self != [plc sourcePeople] && plc != [UnifiedPeopleListController sharedController]) {
			NSArray *presentities = [[plc sourcePeople] people];
			
			for (Presentity *presentity in presentities) {
				if ([[presentity canonicalFormOfID] isEqualToString:[fp8 canonicalFormOfID]]) {
					//Don't remove the presentity if the same user is on another service's contact list
					[[[UnifiedPeopleListController sharedController] sourcePeople] chax_removePresentity:fp8];
					[[[UnifiedPeopleListController sharedController] sourcePeople] chax_addPresentity:presentity];
					
					return [self chax_removePresentity:fp8];
				}
			}
		}
	}
	
	[[[UnifiedPeopleListController sharedController] sourcePeople] chax_removePresentity:fp8];
	return [self chax_removePresentity:fp8];
}

- (BOOL)chax_removePeopleFromArray:(id)fp8
{
	[[[UnifiedPeopleListController sharedController] sourcePeople] chax_removePeopleFromArray:fp8];
	return [self chax_removePeopleFromArray:fp8];
}

@end
