/*
 * Chax_PeopleList.m
 *
 * Copyright (c) 2007-2009 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "Chax_PeopleList.h"
#import "Chax_PeopleListController.h"
#import "Chax.h"
#import "UnifiedPeopleListController.h"

@implementation PeopleList (Chax)

+ (void)load
{
	MethodSwizzle([self class], @selector(displayNameForItem:), @selector(chax_displayNameForItem:));
	MethodSwizzle([self class], @selector(displayStatusForItem:), @selector(chax_displayStatusForItem:));
	MethodSwizzle([self class], @selector(_updateLayout), @selector(chax_updateLayout));
	MethodSwizzle([self class], @selector(endChanges), @selector(chax_endChanges));
	MethodSwizzle([self class], @selector(tableView:groupRowClicked:), @selector(chax_tableView:groupRowClicked:));
	MethodSwizzle([self class], @selector(tableView:validateDrop:proposedRow:proposedDropOperation:), @selector(chax_tableView:validateDrop:proposedRow:proposedDropOperation:));
	MethodSwizzle([self class], @selector(_processPresentityForGroupChange:), @selector(chax_processPresentityForGroupChange:));
	MethodSwizzle([self class], @selector(tableView:deleteRows:), @selector(chax_tableView:deleteRows:));
}

#pragma mark -
#pragma mark Added Methods
#pragma mark -

- (void)chax_updateRowHeights
{
	//This means showing of buddy pictures is being turned off
	if ([Chax boolForKey:@"UseCustomContactListFonts"] && (_viewOptions & 1 == 1)) {
		NSData *data = [Chax dataForKey:@"ContactListFont"];
		NSFont *font = (data != nil) ? [NSUnarchiver unarchiveObjectWithData:data] : nil;
		
		if (font) {
			[_table setRowHeight:(int)([font xHeight] + [font ascender] - [font descender])];
		}
	} else {
		float height = (_viewOptions & 1 == 1) ? 20.0 : 32.0;
		[_table setRowHeight:height];
	}
}

#pragma mark -
#pragma mark Swizzled Methods
#pragma mark -

- (id)chax_displayNameForItem:(id)item
{
	if ([Chax boolForKey:@"UseCustomContactListFonts"]) {
		NSData *data = [Chax objectForKey:@"ContactListFont"];
		if (data) {
			NSFont *font = [NSUnarchiver unarchiveObjectWithData:data];
			if (font) {
				NSMutableAttributedString *name = [[[self chax_displayNameForItem:item] mutableCopy] autorelease];
				[name addAttribute:NSFontAttributeName value:font range:NSMakeRange(0, [[name string] length])];
				return name;
			}
		}
	}
	
	return [self chax_displayNameForItem:item];
}

- (id)chax_displayStatusForItem:(id)item
{
	if ([[self delegate] isKindOfClass:[PeopleListController class]] && [Chax boolForKey:[NSString stringWithFormat:@"%@:%@.HideTextStatus", [[self delegate] name], [[[[self delegate] representedAccount] service] internalName]]]) {
		return [[[NSAttributedString alloc] initWithString:@""] autorelease];
	} else {
		if ([Chax boolForKey:@"UseCustomContactListFonts"]) {
			NSData *data = [Chax objectForKey:@"ContactListStatusFont"];
			if (data) {
				NSFont *font = [NSUnarchiver unarchiveObjectWithData:data];
				if (font) {
					NSMutableAttributedString *message = [[[self chax_displayStatusForItem:item] mutableCopy] autorelease];
					[message addAttribute:NSFontAttributeName value:font range:NSMakeRange(0, [[message string] length])];
					return message;
				}
			}
		}
		return [self chax_displayStatusForItem:item];
	}
}

- (void)chax_updateLayout
{
	[self chax_updateLayout];
	[self chax_updateRowHeights];
}

- (void)chax_endChanges
{
	[self chax_endChanges];
	
	[[NSNotificationCenter defaultCenter] postNotificationName:@"ResizeContactList" object:self];
}

- (void)chax_tableView:(id)fp8 groupRowClicked:(int)fp12
{
	[self chax_tableView:fp8 groupRowClicked:fp12];
	[[NSNotificationCenter defaultCenter] postNotificationName:@"ResizeContactList" object:self];
}

- (NSDragOperation)chax_tableView:(NSTableView *)tableView validateDrop:(id <NSDraggingInfo>)info proposedRow:(int)row proposedDropOperation:(NSTableViewDropOperation)operation
{
	if ([tableView respondsToSelector:@selector(chax_popToFront)]) {
		[tableView performSelector:@selector(chax_popToFront) withObject:nil afterDelay:1.0];
	}
	return [self chax_tableView:tableView validateDrop:info proposedRow:row proposedDropOperation:operation];
}

- (void)chax_processPresentityForGroupChange:(id)fp8
{
	[self chax_processPresentityForGroupChange:fp8];
	
	[[UnifiedPeopleListController sharedController] performSelector:@selector(processPresentityGroupChange:) withObject:fp8 afterDelay:0.0];
}

- (BOOL)chax_tableView:(id)fp8 deleteRows:(id)fp12
{
	//Special case for the unified contact list with groups visible
	if ([[self delegate] isKindOfClass:[UnifiedPeopleListController class]]) {
		return [[self delegate] peopleList:self deletePeople:[self selectedPresentities]];
	}
	
	return [self chax_tableView:fp8 deleteRows:fp12];
}

@end
