/*
 * Chax_PeopleListController.m
 *
 * Copyright (c) 2007-2009 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import <Carbon/Carbon.h>
#import "Chax_PeopleListController.h"
#import "Chax.h"
#import "Chax_PeopleList.h"
#import "UnifiedPeopleListController.h"

@implementation PeopleListController (Chax)

+ (void)load
{
	MethodSwizzle([self class], @selector(dealloc), @selector(chax_dealloc));
	MethodSwizzle([self class], @selector(windowDidLoad), @selector(chax_windowDidLoad));
	MethodSwizzle([self class], @selector(validateMenuItem:), @selector(chax_validateMenuItem:));
	MethodSwizzle([self class], @selector(toggleHidePictures:), @selector(chax_toggleHidePictures:));
}

#pragma mark -
#pragma mark Added Methods
#pragma mark -

- (void)chax_sortByHandle:(id)sender
{
	[[self peopleList] setSortOrder:100];
	[[self peopleList] saveSortOrderToDefaultsAndNotify:NO];
}

- (void)chax_toggleTextStatus:(NSMenuItem *)sender
{
	[sender setState:(([sender state] == NSOnState) ? NSOffState : NSOnState)];
	[Chax setBool:(([sender state] == NSOnState) ? NO : YES) forKey:[NSString stringWithFormat:@"%@:%@.HideTextStatus", [self name], [[[self representedAccount] service] internalName]]];
	[(NSTableView *)[[self peopleList] table] reloadData];
}

- (void)chax_notificationReceived:(NSNotification *)note
{
	if ([[note name] isEqualToString:@"ReloadContactList"]) {
		[(NSTableView *)[[self peopleList] table] reloadData];
		[[self peopleList] chax_updateRowHeights];
		//[self performSelector:@selector(chax_performDelayedUpdate) withObject:nil afterDelay:0.0];
	}/* else if ([[note name] isEqualToString:@"ResizeContactList"]) {
		[self chax_resizeWindow];
	}*/
	
	[self chax_resizeWindow];
}

- (void)chax_performDelayedUpdate
{
	[[self peopleList] chax_updateRowHeights];
	[self chax_resizeWindow];
}

- (void)chax_resizeWindow
{
	if ([_myStatusController currentStatus] > 1 && [Chax boolForKey:@"AutoresizeContactList"] && [_myStatusController currentStatus] > 1 && ![[self representedAccount] justLoggedIn]) {
		NSRect frame = [[self window] frame], screenFrame = [[[self window] screen] frame];
		NSRect preferredFrame = [self windowWillUseStandardFrame:[self window] defaultFrame:screenFrame];
		
		//if the bottom edge of the window is within 20 pixels of the bottom and the top of the buddy list isn't at the top of the screen, resize upwards if possible
		//otherwise resize down as usual
		if (frame.origin.y < 20 && (screenFrame.size.height - GetMBarHeight() - (frame.origin.y + frame.size.height) > 10)) {
			//we're within 30 pixels of the bottom of the screen, this means we should resize upward
			preferredFrame.origin.y = frame.origin.y;
		} else {
			preferredFrame.origin.y -= 1;
		}
		
		preferredFrame.size.width = [[self window] frame].size.width;
		preferredFrame.origin.x = [[self window] frame].origin.x;
		preferredFrame.size.height += 1;
		
		[[self window] setFrame:preferredFrame display:YES animate:YES];
	}
}

#pragma mark -
#pragma mark Swizzled Methods
#pragma mark -

- (void)chax_dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[self chax_dealloc];
}

- (void)chax_windowDidLoad
{
	[self chax_windowDidLoad];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(chax_notificationReceived:) name:@"ReloadContactList" object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(chax_notificationReceived:) name:@"ResizeContactList" object:[self peopleList]];
	
	if ([Chax boolForKey:@"HideContactListsWhenInactive"]) {
		[[self window] setHidesOnDeactivate:YES];
	}
}

- (BOOL)chax_validateMenuItem:(NSMenuItem *)sender
{
	if ([sender tag] == SHOW_TEXT_STATUS_MENU_ITEM) {
		[sender setState:([Chax boolForKey:[NSString stringWithFormat:@"%@:%@.HideTextStatus", [self name], [[[self representedAccount] service] internalName]]] ? NSOffState : NSOnState)];
	} else if ([sender tag] == BY_HANDLE_MENU_ITEM) {
		[sender setState:[[self peopleList] sortOrder] == 100 ? NSOnState : NSOffState];
	}
	
	return [self chax_validateMenuItem:sender];
}

- (void)chax_toggleHidePictures:(id)fp8
{
	[self chax_toggleHidePictures:fp8];
	[[self peopleList] chax_updateRowHeights];
	[self chax_resizeWindow];
}

@end
