/*
 * Chax_SecureWindow.m
 *
 * Copyright (c) 2007-2009 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "Chax_SecureWindow.h"
#import "Chax.h"

BOOL _chax_blockNextShowWindow = NO;

@implementation Chax_SecureWindow

+ (void)chax_blockNextShowWindow
{
	_chax_blockNextShowWindow = YES;
}

- (void)chax_toggleAlwaysOnTop:(id)sender
{
	[self setLevel:([self level] == NSNormalWindowLevel) ? NSStatusWindowLevel : NSNormalWindowLevel];
}

-(void)swipeWithEvent:(NSEvent *)event
{
	if ([[[self delegate] class] isEqual:[ChatWindowController class]]) {
		CGFloat deltaX = [event deltaX];
		CGFloat deltaY = [event deltaY];
		
		if (deltaX > 0 || deltaY > 0) {
			[[self delegate] gotoPreviousChat:nil];
		} else if (deltaX < 0 || deltaY < 0) {
			[[self delegate] gotoNextChat:nil];
		}
	}
}

- (void)makeKeyAndOrderFront:(id)sender
{
	if (![ChatWindowController wantsCollectedChats] && [Chax boolForKey:@"SaveWindowFrames"] && [[[self delegate] class] isEqual:[ChatWindowController class]]) {
		//Read saved presentity preferences and check to see if there is a saved frame to set to
		NSDictionary *presentities = [Chax objectForKey:@"CustomPresentities"], *info;
		NSString *key = [[[[[self delegate] currentChat] otherPresentity] abPerson] uniqueId], *userID = [[[[[self delegate] currentChat] otherPresentity] ID] lowercaseString];
		if (key == nil) {
			key = userID;
		}
		
		info = [presentities objectForKey:key];
		
		if (info) {
			[self setFrameFromString:[info objectForKey:[@"WindowFrame." stringByAppendingString:userID]]];
		}
	}
	
	if (_chax_blockNextShowWindow) {
		_chax_blockNextShowWindow = NO;
		return;
	}
	
	[super makeKeyAndOrderFront:sender];
}

- (BOOL)validateMenuItem:(NSMenuItem *)sender
{
	if ([sender tag] == ALWAYS_ON_TOP_MENU_ITEM && [self delegate] && ([[[self delegate] class] isEqual:[ChatWindowController class]] || [[[self delegate] class] isEqual:[VideoChatController class]] || [[[self delegate] class] isEqual:[AudioChatController class]])) {
		[sender setState:([self level] == NSNormalWindowLevel) ? NSOffState : NSOnState];
		return YES;
	}
	return [super validateMenuItem:sender];
}

@end
