/*
 * Controller.m
 *
 * Copyright (c) 2007-2009 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "Controller.h"
#import <Security/Security.h>
#import <unistd.h>
#import <sys/socket.h>

#import <stdio.h>
#import <grp.h>
#import <pwd.h>
#import <sys/stat.h>

#define DONATE_URL [NSURL URLWithString:@"http://www.ksuther.com/chax/donate"]
#define FAQ_URL [NSURL URLWithString:@"http://www.ksuther.com/chax/faq"]

@implementation Controller

- (void)awakeFromNib
{
	[_removeButton setTitle:NSLocalizedString(@"remove", nil)];
	[_readMeButton setTitle:NSLocalizedString(@"open_read_me", nil)];
	
	[self updateInstallationInfo];
	[_window center];
	[NSApp activateIgnoringOtherApps:YES];
	[_window makeKeyAndOrderFront:nil];
}

- (void)applicationDidFinishLaunching:(NSNotification *)note
{
	long version;
	
	if ((Gestalt(gestaltSystemVersion, &version) == noErr) && ((version < 4163) || (version > 4192))) {
		NSBeginAlertSheet(NSLocalizedString(@"version_title", nil), NSLocalizedString(@"quit", nil), nil, nil, _window, self, @selector(versionSheetDidEnd:returnCode:contextInfo:), NULL, nil, NSLocalizedString(@"version_msg", nil), nil);
	}
	
	struct group *wheelInfo = getgrnam("wheel");
	char *member = *(wheelInfo->gr_mem++);
	
	while (member != 0) {
		if ([[NSString stringWithCString:member] isEqualToString:NSUserName()]) {
			NSBeginAlertSheet(NSLocalizedString(@"wheel_title", nil), @"OK", NSLocalizedString(@"view_faq", nil), nil, _window, self, @selector(wheelSheetDidEnd:returnCode:contextInfo:), NULL, nil, NSLocalizedString(@"wheel_msg", nil), nil);
		}
		member = *(wheelInfo->gr_mem++);
	}
}

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)application
{
	return YES;
}

- (IBAction)showReadMe:(id)sender
{
	[[NSWorkspace sharedWorkspace] openFile:[[NSBundle mainBundle] pathForResource:@"Chax Read Me" ofType:@"rtf"] withApplication:@"TextEdit.app"];
}

- (void)updateInstallationInfo
{
	NSBundle *bundle = nil;
	NSString *version = [[NSBundle mainBundle] objectForInfoDictionaryKey:@"CFBundleVersion"];
	NSString *installTitle = [NSString stringWithFormat:NSLocalizedString(@"install_title", nil), version];
	
	[_window setTitle:installTitle];
	[_installTitle setStringValue:installTitle];
	[_installText setStringValue:[NSString stringWithFormat:NSLocalizedString(@"install_msg", nil), version]];
	[_installButton setTitle:NSLocalizedString(@"install", nil)];
	[_removeButton setEnabled:NO];
	
	//Check if we're installed
	if ([[NSFileManager defaultManager] fileExistsAtPath:@"/Library/Application Support/SIMBL/Plugins/Chax.bundle"] ||
			[[NSFileManager defaultManager] fileExistsAtPath:[@"~/Library/Application Support/SIMBL/Plugins/Chax.bundle" stringByExpandingTildeInPath]] ||
			(bundle = [NSBundle bundleWithPath:[@"~/Library/InputManagers/Chax/Chax.bundle" stringByExpandingTildeInPath]])) {
		NSString *message = [NSString stringWithFormat:NSLocalizedString(@"installed_already_installed", nil), version];
		[_installText setStringValue:message];
	} else if ([[NSFileManager defaultManager] fileExistsAtPath:@"/Library/InputManagers/Chax/Chax.bundle"] &&
			(bundle = [NSBundle bundleWithPath:@"/Library/InputManagers/Chax/Chax.bundle"])) {
		NSString *message, *buttonText;
		
		if (![[bundle objectForInfoDictionaryKey:@"CFBundleVersion"] isEqualToString:[[NSBundle mainBundle] objectForInfoDictionaryKey:@"CFBundleVersion"]]) {
			buttonText = NSLocalizedString(@"update", nil);
			message = [NSString stringWithFormat:NSLocalizedString(@"installed_update_msg", nil), version];
		} else {
			buttonText = NSLocalizedString(@"reinstall", nil);
			message = [NSString stringWithFormat:NSLocalizedString(@"installed_reinstall_msg", nil), version];
		}
		
		[_installButton setTitle:buttonText];
		[_installText setStringValue:message];
		[_removeButton setEnabled:YES];
	}
}

- (NSString *)executeInstallerToolWithArgument:(char *)arg
{
	AuthorizationRef authorizationRef;
	AuthorizationItem items[1];
	AuthorizationRights rights;
	char *args[2];
	FILE *pipe;
	OSStatus err;
	
	err = AuthorizationCreate(NULL, kAuthorizationEmptyEnvironment, kAuthorizationFlagDefaults, &authorizationRef);
	if (err != noErr) {
		[self throwError:@"auth_error"];
		return nil;
	}
	
	items[0].name = kAuthorizationRightExecute;
	items[0].value = NULL;
	items[0].valueLength = 0;
	items[0].flags = 0;
	
	args[0] = arg;
	args[1] = NULL;
	
	rights.count = 1;
	rights.items = items;
	
	err = AuthorizationCopyRights(authorizationRef, &rights, kAuthorizationEmptyEnvironment, kAuthorizationFlagInteractionAllowed | kAuthorizationFlagExtendRights, NULL);
	if (err != noErr) {
		//Don't throw an error because this means the user cancelled entering a password
		return nil;
	}
	
	err = AuthorizationExecuteWithPrivileges(authorizationRef, [[[NSBundle mainBundle] pathForResource:@"install_tool" ofType:nil] fileSystemRepresentation], 0, args, &pipe);
	if (err != noErr) {
		[self throwError:@"auth_error"];
		return nil;
	}
	
	err = AuthorizationFree(authorizationRef, kAuthorizationFlagDestroyRights);
	if (err != noErr) {
		[self throwError:@"auth_error"];
		return nil;
	}
	
	NSMutableString *resultString = [NSMutableString string];
	
	//Read the result of the installer tool
	while (!feof(pipe)) {
		[resultString appendFormat:@"%c", fgetc(pipe)];
	}
	
	[resultString deleteCharactersInRange:NSMakeRange([resultString length] - 1, 1)];
	
	fclose(pipe);
	
	return resultString;
}

- (IBAction)install:(id)sender
{
	NSString *resultString = [self executeInstallerToolWithArgument:"i"];
	
	if (resultString) {
		if ([resultString isEqualToString:@"success_installing"]) {
			NSBeginAlertSheet(NSLocalizedString(@"success_title", nil), NSLocalizedString(@"donate", nil), NSLocalizedString(@"quit", nil), @"OK", _window, self, NULL, @selector(installedSheetDidEnd:returnCode:contextInfo:), nil, NSLocalizedString(@"success_msg", nil), nil);
			[self updateInstallationInfo];
		} else {
			NSBeginAlertSheet(NSLocalizedString(@"error_title", nil), nil, nil, nil, _window, self, NULL, @selector(installedSheetDidEnd:returnCode:contextInfo:), nil, NSLocalizedString(resultString, nil), nil);
		}
	}
}

- (IBAction)remove:(id)sender
{
	NSString *resultString = [self executeInstallerToolWithArgument:"d"];
	
	if (resultString) {
		if ([resultString isEqualToString:@"success_removing"]) {
			NSBeginAlertSheet(NSLocalizedString(@"remove_title", nil), nil, nil, nil, _window, self, NULL, NULL, nil, NSLocalizedString(@"remove_msg", nil), nil);
		} else {
			NSBeginAlertSheet(NSLocalizedString(@"remove_error_title", nil), nil, nil, nil, _window, self, NULL, NULL, nil, NSLocalizedString(@"remove_error_msg", nil), nil);
		}
	}
	
	[self updateInstallationInfo];
}

- (void)installedSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	if (returnCode == NSOKButton) {
		[[NSWorkspace sharedWorkspace] openURL:DONATE_URL];
	} else if (returnCode == NSAlertAlternateReturn) {
		[NSApp terminate:nil];
	}
}

- (void)versionSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	[NSApp terminate:nil];
}

- (void)wheelSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	if (returnCode == NSCancelButton) {
		[[NSWorkspace sharedWorkspace] openURL:FAQ_URL];
	}
}

- (void)throwError:(NSString *)message
{
	NSBeginAlertSheet(NSLocalizedString(@"error_title", nil), nil, nil, nil, _window, self, NULL, NULL, nil, NSLocalizedString(message, nil), nil);
}

@end