/*
 * UnifiedPeopleListController.m
 *
 * Copyright (c) 2007-2009 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "UnifiedPeopleListController.h"
#import "UnifiedAccount.h"
#import "Chax.h"

@implementation UnifiedPeopleListController

+ (UnifiedPeopleListController *)sharedController
{
	static UnifiedPeopleListController *_sharedController = nil;
	
	if (!_sharedController) {
		AbstractAccount *account = [[UnifiedAccount alloc] initWithUniqueID:@"ChaxUnified"];
		
		[account setAccountLoginStatus:4];
		[account setString:ChaxLocalizedString(@"all_accounts") forKey:@"LoginAs"];
		
		_sharedController = [[UnifiedPeopleListController alloc] initWithAccount:account];
	}
	
	return _sharedController;
}

- (id)initWithAccount:(AbstractAccount *)account
{
	if ([super initWithAccount:account]) {
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(chax_notificationReceived:) name:@"ReloadContactList" object:nil];
		
		[self setPrefIdentifier:@"Chax"];
		[self setName:ChaxLocalizedString(@"contacts")];
	}
	return self;
}

- (void)dealloc
{
	[_addMenu release];
	[_addGroupString release];
	[_editGroupsString release];
	[super dealloc];
}

- (void)windowDidLoad
{
	[super windowDidLoad];
	
	NSMenu *addButtonMenu = [_addButton menu];
	
	_addGroupString = [[[addButtonMenu itemAtIndex:1] title] copy];
	_editGroupsString = [[[addButtonMenu itemAtIndex:3] title] copy];
	
	if (!_addGroupString) {
		_addGroupString = [[NSString alloc] initWithString:@"Add Group..."];
	}
	
	if (!_editGroupsString) {
		_editGroupsString = [[NSString alloc] initWithString:@"Edit Groups..."];
	}
}

- (void)windowDidMove:(id)fp8
{
	NSRect frame = [[self window] frame];
	frame.origin.y += frame.size.height;
	
	if (![self isTableCollapsed]) {
		[[NSUserDefaults standardUserDefaults] setObject:NSStringFromRect(frame) forKey:@"Chax.WindowSize"];
	}
	
	[super windowDidMove:fp8];
}

- (void)windowDidResize:(id)fp8
{
	NSRect frame = [[self window] frame];
	frame.origin.y += frame.size.height;
	
	if (![self isTableCollapsed]) {
		[[NSUserDefaults standardUserDefaults] setObject:NSStringFromRect(frame) forKey:@"Chax.WindowSize"];
	}
	
	[super windowDidResize:fp8];
}

- (BOOL)supportsOfflineToggle
{
	return YES;
}

- (BOOL)supportsGroups
{
	return YES;
}

- (BOOL)peopleList:(id)fp8 canRemoveRows:(id)fp12
{
	return YES;
}

- (void)setGroupsSortOrder:(id)fp8
{
	[super setGroupsSortOrder:fp8];
	
	[self refreshList];
}

- (void)logServiceInOrOut:(id)sender
{
	if ([[[AccountController sharedInstance] allConnectedAccounts] count] == 0) {
		NSArray *accounts = [[AccountController sharedInstance] allActiveAccounts];
		
		for (AbstractAccount *account in accounts) {
			if ([account autoLogin]) {
				[Service autoLoginAllServices];
				[[self representedAccount] setAccountLoginStatus:4];
				return;
			}
		}
		
		[[self representedAccount] setAccountLoginStatus:0];
	} else {
		[Service logoutAllServices];
		[[self representedAccount] setAccountLoginStatus:0];
	}
}

- (BOOL)validateMenuItem:(NSMenuItem *)menuItem
{
	if ([menuItem action] == @selector(logServiceInOrOut:)) {
		if ([[[AccountController sharedInstance] allConnectedAccounts] count] == 0) {
			[menuItem setTitle:[NSString stringWithFormat:NSLocalizedString(@"Log In to %@", @"Log In to %@"), [[self representedAccount] login]]];
		} else {
			[menuItem setTitle:[NSString stringWithFormat:NSLocalizedString(@"Log Out of %@", @"Log Out of %@"), [[self representedAccount] login]]];
		}
		return YES;
	}
	
	return [super validateMenuItem:menuItem];
}

- (void)processPresentityGroupChange:(Presentity *)presentity
{
	[self reloadContacts];
}

- (void)_arrangesByGroupChanged
{
	[super _arrangesByGroupChanged];
	
	if (!_addMenu) {
		//Build the menu delayed the first time
		[self performSelector:@selector(rebuildAddBuddyMenu) withObject:nil afterDelay:2.0];
	} else {
		[self rebuildAddBuddyMenu];
	}
}

- (void)reloadContacts
{
	[_sourcePeople beginCoalescedChanges];
	
	[[self peopleList] removeAllPresentitiesAndGroups:YES];
	
	for (PeopleListController *pl in [PeopleListController peopleListControllers]) {
		if (pl != self) {
			NSArray *presentities = [[pl peopleList] allPresentities];
			
			for (Presentity *presentity in presentities) {
				[_sourcePeople addPresentity:presentity];
			}
		}
	}
	
	[_sourcePeople endCoalescedChanges];
}

- (void)rebuildAddBuddyMenu
{
	if (!_addMenu) {
		_addMenu = [[NSMenu alloc] init];
	}
	
	NSMenu *addButtonMenu = [_addButton menu];
	
	if (!_addGroupString) {
		_addGroupString = [[[addButtonMenu itemAtIndex:1] title] copy];
		
		if (!_addGroupString) {
			_addGroupString = [[NSString alloc] initWithString:@"Add Group..."];
		}
	}
	
	if (!_editGroupsString) {
		_editGroupsString = [[[addButtonMenu itemAtIndex:3] title] copy];
		
		if (!_editGroupsString) {
			_editGroupsString = [[NSString alloc] initWithString:@"Edit Groups..."];
		}
	}
	
	while (_addMenu.numberOfItems > 0) {
		[_addMenu removeItemAtIndex:0];
	}
	
	NSArray *connectedAccounts = [[AccountController sharedInstance] allConnectedAccounts];
	
	for (AbstractAccount *nextAccount in connectedAccounts) {
		NSString *description = [nextAccount description];
		
		if (description) {
			NSMenu *accountSubmenu = [[[NSMenu alloc] init] autorelease];
			
			[[_addMenu addItemWithTitle:description action:nil keyEquivalent:@""] setSubmenu:accountSubmenu];
			
			[[accountSubmenu addItemWithTitle:NSLocalizedString(@"Add Buddy\\U2026", nil) action:@selector(addABuddyChax:) keyEquivalent:@""] setRepresentedObject:[nextAccount uniqueID]];
			[[accountSubmenu addItemWithTitle:_addGroupString action:@selector(addAGroupChax:) keyEquivalent:@""] setRepresentedObject:[nextAccount uniqueID]];
			[accountSubmenu addItem:[NSMenuItem separatorItem]];
			[[accountSubmenu addItemWithTitle:_editGroupsString action:@selector(editGroupsChax:) keyEquivalent:@""] setRepresentedObject:[nextAccount uniqueID]];
		}
	}
	
	[_addButton setUsesMenu:YES];
	[_addButton setMenu:_addMenu];
}

- (void)addABuddyChax:(id)sender
{
	AbstractAccount *account = [[AccountController sharedInstance] accountForUniqueID:[sender representedObject]];
	PeopleListController *accountPeopleListController = [PeopleListController peopleListControllerWithRepresentedAccount:account];
	
	AddBuddy *addBuddy = [accountPeopleListController addBuddy];
	
	id groups = [accountPeopleListController groupsWithCapability:nil];
	
	[addBuddy openForNewPresentityWithGroups:groups buddyWindow:[self window]];
}

- (void)addAGroupChax:(id)sender
{
	AbstractAccount *account = [[AccountController sharedInstance] accountForUniqueID:[sender representedObject]];
	PeopleListController *accountPeopleListController = [PeopleListController peopleListControllerWithRepresentedAccount:account];
	
	[accountPeopleListController showWindow:nil];
	[GroupsEditorController addGroupForPeopleListController:accountPeopleListController];
}

- (void)editGroupsChax:(id)sender
{
	AbstractAccount *account = [[AccountController sharedInstance] accountForUniqueID:[sender representedObject]];
	PeopleListController *accountPeopleListController = [PeopleListController peopleListControllerWithRepresentedAccount:account];
	
	[accountPeopleListController showWindow:nil];
	[GroupsEditorController editGroupsForPeopleListController:accountPeopleListController];
}

@end
